package generate

import (
	"fmt"
	"git.168cad.top/zhengqiuyun/rym-util/a/conf"
	"github.com/iancoleman/strcase"
	"io"
	"os"
	"regexp"
	"strings"
)

type Table struct {
	Name        string
	Description string
}

type TableField struct {
	FieldName  string
	FieldType  string
	Length     int16
	LengthVar  int32
	Notnull    bool
	Comment    string
	ColumnType string
}

//ProjectName 项目名称
const ProjectName = "git.168cad.top/zhengqiuyun/rym-util"
const PathSeparator = "\\"
const BasePackage = "pay"
const ModelReplace = true
const webPackageName = "web"
const servicePackageName = "service"
const daoPackageName = "dao"
const modelPackageName = "model"

func BuildPgModels() {
	var tables []Table
	if DBType == "PostgreSql" {
		tables = AllTables()
	} else {
		tables = MysqlAllTables()
	}
	if len(DoTables) > 0 {
		for i := 0; i < len(tables); i++ {
			table := tables[i]
			doTable, ok := DoTables[table.Name]
			if ok {
				var tableFields []TableField
				if conf.DBType == "PostgreSql" {
					tableFields = GetTableFiledByTableName(table.Name)
				} else {
					tableFields = MysqlGetTableFiledByTableName(table.Name)
				}
				if len(tableFields) > 0 {
					generateModel(&table, tableFields, doTable)
					if doTable.GenerateApiParams {
						generateApiParams(&table, tableFields, doTable)
					}
					if doTable.GenerateResponse {
						generateResponse(&table, doTable)
					}
					if doTable.GenerateDao {
						generateDao(&table, tableFields, doTable)
					}
					if doTable.GenerateService {
						generateService(&table, tableFields, doTable)
					}
					if doTable.GenerateApi {
						generateApi(&table, tableFields, doTable)
					}
				}
			}

		}
	}
}

func getModelImport(field TableField) string {
	return ModelImportMap[field.FieldType]
}

func getModelFiledType(field TableField) string {
	if conf.DBType == "PostgreSql" {
		return DbToGoPG[field.FieldType]
	} else {
		if strings.Index(field.ColumnType, "unsigned") > 0 {
			return "u" + DbToGoMySql[field.FieldType]
		} else {
			return DbToGoMySql[field.FieldType]
		}

	}

}

func getModelFieldOrm(field TableField, doTable TableInfo) string {
	if strings.Contains(strings.ToLower(doTable.PK), strings.ToLower(field.FieldName)) {
		return `gorm:"primaryKey:` + field.FieldName + `"`
	} else {
		return `gorm:"column:` + field.FieldName + `"`
	}
}

//获取字段json描述
func getModelFieldJson(field TableField) string {
	return `json:"` + strcase.ToLowerCamel(strings.ToLower(field.FieldName)) + `,omitempty"`
}

//获取字段说明
func getModelFieldComment(field TableField) string {
	if len(field.Comment) > 0 {
		return "// " + field.Comment
	}
	return ""
}

func getModelName(tableName string, doTable TableInfo) string {
	if doTable.ModelName == "" {
		return strcase.ToCamel(strings.ToLower(tableName))
	} else {
		return doTable.ModelName
	}
}

//生成Model
func generateModel(table *Table, fields []TableField, doTable TableInfo) {
	modelName := getModelName(table.Name, doTable)
	packageStr := "package " + modelPackageName + "\n\n"

	importPs := mapset.NewSet()

	content := ""
	//表注释
	if len(table.Description) > 0 {
		content += "// " + modelName + " " + table.Description + "\n"
	}
	content += "type " + modelName + " struct {\n"
	columnsStruct := "type " + modelName + "ColumnsStruct struct {\n"
	columns := "var " + modelName + "Columns = " + modelName + "ColumnsStruct{\n"
	//生成字段
	for _, field := range fields {
		//字段名
		fieldName := strcase.ToCamel(strings.ToLower(field.FieldName))
		//根据字段 获取对应的字段的类型
		fieldType := getModelFiledType(field)
		//根据字段 获取对应的字段的Orm 表字段信息
		fieldOrm := getModelFieldOrm(field, doTable)
		//根据字段 获取对应的JSON tag 信息
		fieldJson := getModelFieldJson(field)
		//根据字段  获取对应的备注信息
		fieldComment := getModelFieldComment(field)

		importP := getModelImport(field)
		if importP != "" && !importPs.Contains(importP) {
			importPs.Add(importP)
		}
		content += "\t" + fieldName + " *" + fieldType + " `" + fieldOrm + " " + fieldJson + "` " + fieldComment + "\n"
		columnsStruct += "\t" + fieldName + " string\n"
		columns += "\t" + fieldName + " : \"" + field.FieldName + "\",\n"
	}
	content += "}\n"
	columnsStruct += "}\n"
	columns += "}\n"

	importStr := ""
	if importPs.Cardinality() > 0 {
		importStr = "import (\n"
		for val := range importPs.Iterator().C {
			importStr += "	\"" + (val).(string) + "\"\n"
		}
		importStr += ")\n\n"
	}

	//表名
	tableName := "func (" + modelName + ") TableName() string {\n   return \"" + table.Name + "\"\n}\n\n"
	fileContent := packageStr + importStr + tableName + content + columnsStruct + columns
	createOrReplaceFile(BasePackage, modelPackageName, doTable.Group, "", modelName, fileContent)
}

//生成dao
func generateDao(table *Table, fields []TableField, doTable TableInfo) {
	projectName := ProjectName
	modelName := getModelName(table.Name, doTable)
	packageStr := "package " + daoPackageName + "\n\n"

	haveCreateAt := false
	haveUpdateAt := false
	var pks []TableField
	updateTimeFieldName := strcase.ToCamel(strings.ToLower(UpdateTime))
	createTimeFieldName := strcase.ToCamel(strings.ToLower(CreateTime))
	orderBy := make(map[string]string)
	for j := 0; j < len(fields); j++ {
		field := fields[j]
		if strings.Contains(doTable.PK, field.FieldName) {
			pks = append(pks, field)
		}
		if strings.ToLower(field.FieldName) == CreateTime {
			orderBy[field.FieldName] = "DESC"
			haveCreateAt = true
		}
		if strings.ToLower(field.FieldName) == UpdateTime {
			haveUpdateAt = true
		}
	}
	create := ""
	if doTable.Group == "" {
		create += "func Create" + modelName + "(m *model." + modelName + ") *model." + modelName + " {\n"
	} else {
		create += "func Create" + modelName + "(m *" + doTable.Group + "Model." + modelName + ") *" + doTable.Group + "Model." + modelName + " {\n"
	}

	if haveCreateAt || haveUpdateAt {
		create += "\tvar now = time.Now()\n"
	}
	if haveCreateAt {
		create += "\tm." + createTimeFieldName + " = &now\n"
	}
	if haveUpdateAt {
		create += "\tm." + updateTimeFieldName + " = &now\n"
	}
	create += "\tresult := db.GetPgDb().Create(&m)\n"
	create += "\tif result.Error != nil {\n"
	create += "\t\texception.ThrowsErr(result.Error)\n"
	create += "\t}\n"
	create += "\treturn m\n"
	create += "}\n"
	updateByPrimaryKey := ""
	findByPrimaryKey := ""
	if len(pks) > 0 {
		pkParams := ""
		pkFields := ""
		for j := 0; j < len(pks); j++ {
			field := pks[j]
			pkParams += strcase.ToLowerCamel(strings.ToLower(field.FieldName)) + " " + getModelFiledType(field) + ", "
			pkFields += strcase.ToCamel(field.FieldName) + ": &" + strcase.ToLowerCamel(strings.ToLower(field.FieldName)) + ", "
			orderBy[field.FieldName] = "DESC"
		}
		pkParams = pkParams[0 : len(pkParams)-2]
		pkFields = pkFields[0 : len(pkFields)-2]
		if doTable.Group == "" {
			findByPrimaryKey += "func Find" + modelName + "ByPrimaryKey(" + pkParams + ") (m *model." + modelName + ") {\n"
			findByPrimaryKey += "\tresult := db.GetPgDb().Where(model." + modelName + "{" + pkFields + "}).First(&m)\n"
		} else {
			findByPrimaryKey += "func Find" + modelName + "ByPrimaryKey(" + pkParams + ") (m *" + doTable.Group + "Model." + modelName + ") {\n"
			findByPrimaryKey += "\tresult := db.GetPgDb().Where(" + doTable.Group + "Model." + modelName + "{" + pkFields + "}).First(&m)\n"
		}
		findByPrimaryKey += "\tif result.Error != nil {\n"
		findByPrimaryKey += "\t\tif errors.Is(result.Error, gorm.ErrRecordNotFound) {\n"
		findByPrimaryKey += "\t\t\treturn nil\n"
		findByPrimaryKey += "\t\t} else {\n"
		findByPrimaryKey += "\t\t\texception.ThrowsErr(result.Error)\n"
		findByPrimaryKey += "\t\t}\n"
		findByPrimaryKey += "\t}\n"
		findByPrimaryKey += "\treturn m\n"
		findByPrimaryKey += "}\n"
		if doTable.Group == "" {
			updateByPrimaryKey += "func Update" + modelName + "ByPrimaryKey(m *model." + modelName + ") int64 {\n"
		} else {
			updateByPrimaryKey += "func Update" + modelName + "ByPrimaryKey(m *" + doTable.Group + "Model." + modelName + ") int64 {\n"
		}
		if haveUpdateAt {
			updateByPrimaryKey += "\tvar now = time.Now()\n"
			updateByPrimaryKey += "\tm." + updateTimeFieldName + " = &now\n"
		}
		updateByPrimaryKey += "\tresult := db.GetPgDb().Updates(m)\n"
		updateByPrimaryKey += "\tif result.Error != nil {\n"
		updateByPrimaryKey += "\t\texception.ThrowsErr(result.Error)\n"
		updateByPrimaryKey += "\t}\n"
		updateByPrimaryKey += "\treturn result.RowsAffected\n"
		updateByPrimaryKey += "}\n"
	}
	pageQuery := "func Page" + modelName + "(params []page.Param,pageNum int,pageSize int) *page.Data {\n"
	orderByP := ""
	if len(orderBy) > 0 {
		orderByStr := ""
		for k, v := range orderBy {
			orderByStr += " + model." + modelName + "Columns." + strcase.ToCamel(k) + " + \" " + v + ",\""
		}
		pageQuery += "\torderBy := " + orderByStr[3:len(orderByStr)-2] + "\"\n"
		orderByP = ", orderBy"
	} else {
		pageQuery += "\torderBy := \"\""
	}

	if doTable.Group == "" {
		pageQuery += "\treturn PagePlus.Page(params,pageNum,pageSize,&model." + modelName + "{},[] model." + modelName + "{}" + orderByP + ",db.GetPgDb())\n"
	} else {
		pageQuery += "\treturn PagePlus.Page(params,pageNum,pageSize,&" + doTable.Group + "." + modelName + "{},[]" + doTable.Group + "." + modelName + "{}" + orderByP + ",db.GetPgDb())\n"
	}

	pageQuery += "}\n"
	importStr := "import (\n"
	importStr += "\t\"" + projectName + "/a/PagePlus\"\n"
	if conf.DBType == "" {
		importStr += "\t\"" + projectName + "/a/db\"\n"
	} else {
		importStr += "\t\"" + projectName + "/a/db/" + conf.DBType + "\"\n"
	}
	importStr += "\t\"" + projectName + "/a/exception\"\n"
	importStr += "\t\"" + projectName + "/a/page\"\n"
	if doTable.Group == "" {
		importStr += "\t\"" + projectName + "/" + BasePackage + "/" + modelPackageName + "\"\n"
	} else {
		importStr += "\t" + doTable.Group + "Model \"" + projectName + "/" + BasePackage + "/" + modelPackageName + "/" + doTable.Group + "\"\n"
	}
	if len(pks) > 0 {
		importStr += "\t\"errors\"\n"
		importStr += "\t\"gorm.io/gorm\"\n"
	}
	if haveCreateAt || haveUpdateAt {
		importStr += "\t\"time\"\n"
	}
	importStr += ")\n"
	fileContent := packageStr + importStr + create + updateByPrimaryKey + findByPrimaryKey + pageQuery
	createOrReplaceFile(BasePackage, daoPackageName, doTable.Group, "", modelName, fileContent)
}

//生成Api相关params
func generateApiParams(table *Table, fields []TableField, doTable TableInfo) {
	projectName := ProjectName
	modelName := getModelName(table.Name, doTable)
	packageStr := "package params\n\n"

	importStr := ""

	createParams := "type " + modelName + "AddParams struct {\n"

	updateParams := "type " + modelName + "UpdateParams struct {\n"

	pageParams := "type " + modelName + "PageParams struct {\n" +
		"\tpage.ParamPage\n"
	importPs := mapset.NewSet(projectName + "/a/page")
	for _, field := range fields {
		//字段名
		fieldName := strcase.ToCamel(strings.ToLower(field.FieldName))
		//根据字段 获取对应的字段的类型
		fieldType := getModelFiledType(field)
		//根据字段 获取对应的JSON tag 信息
		fieldJson := getModelFieldJson(field)
		//根据字段  获取对应的备注信息
		fieldComment := getModelFieldComment(field)
		content := ""
		content = "\t" + fieldName + " *" + fieldType + " `" + fieldJson + "` " + fieldComment + "\n"
		if strings.Contains(doTable.PK, field.FieldName) {
			if !doTable.AutoGenerateID {
				createParams += content
			}
			updateParams += content
			pageParams += content
		} else if strings.ToLower(field.FieldName) == Handler ||
			strings.ToLower(field.FieldName) == CreateTime ||
			strings.ToLower(field.FieldName) == UpdateTime {
		} else {
			importP := getModelImport(field)
			if importP != "" && !importPs.Contains(importP) {
				importPs.Add(importP)
			}
			createParams += content
			updateParams += content
			pageParams += content
		}
	}
	createParams += "}\n"
	updateParams += "}\n"
	pageParams += "}\n"

	if importPs.Cardinality() > 0 {
		importStr = "import (\n"
		for val := range importPs.Iterator().C {
			importStr += "	\"" + (val).(string) + "\"\n"
		}
		importStr += ")\n\n"
	}

	fileContent := packageStr + importStr + createParams + updateParams + pageParams
	createOrReplaceFile(BasePackage, webPackageName, doTable.Group, "params", modelName, fileContent)
}

//生成service
func generateService(table *Table, fields []TableField, doTable TableInfo) {
	projectName := ProjectName
	modelName := getModelName(table.Name, doTable)
	packageStr := "package " + servicePackageName + "\n\n"
	haveHandler := false
	for _, field := range fields {
		if strings.ToLower(field.FieldName) == Handler {
			haveHandler = true
		}
	}
	importStr := "import (\n"
	importStr += "\t\"" + projectName + "/a/page\"\n"
	if haveHandler {
		importStr += "\t\"" + projectName + "/a/request\"\n"
	}
	importStr += "\t\"" + projectName + "/a/sqlOp\"\n"

	if doTable.Group == "" {
		importStr += "\t\"" + projectName + "/" + BasePackage + "/" + daoPackageName + "\"\n"
		importStr += "\t\"" + projectName + "/" + BasePackage + "/" + modelPackageName + "\"\n"
		importStr += "\t\"" + projectName + "/" + BasePackage + "/" + webPackageName + "/params\"\n"
	} else {
		importStr += "\t" + doTable.Group + "Dao \"" + projectName + "/" + BasePackage + "/" + daoPackageName + "/" + doTable.Group + "\"\n"
		importStr += "\t" + doTable.Group + "Model \"" + projectName + "/" + BasePackage + "/" + modelPackageName + "/" + doTable.Group + "\"\n"
		importStr += "\t\"" + projectName + "/" + BasePackage + "/" + webPackageName + "/" + doTable.Group + "/params\"\n"
	}
	if haveHandler {
		importStr += "\t\"fmt\"\n"
		importStr += "\t\"github.com/gin-gonic/gin\"\n"
	}
	importStr += ")\n"

	addOne := ""
	if haveHandler {
		addOne += "func Add" + modelName + "(addParams params." + modelName + "AddParams, c *gin.Context) {\n"
	} else {
		addOne += "func Add" + modelName + "(addParams params." + modelName + "AddParams) {\n"
	}
	if doTable.Group == "" {
		addOne += "\tvar m model." + modelName + "\n"
	} else {
		addOne += "\tvar m " + doTable.Group + "Model." + modelName + "\n"
	}
	updateByPrimaryKey := ""
	if haveHandler {
		updateByPrimaryKey += "func Update" + modelName + "(updateParams params." + modelName + "UpdateParams, c *gin.Context) {\n"
	} else {
		updateByPrimaryKey += "func Update" + modelName + "(updateParams params." + modelName + "UpdateParams) {\n"
	}
	if doTable.Group == "" {
		updateByPrimaryKey += "\tvar m model." + modelName + "\n"
	} else {
		updateByPrimaryKey += "\tvar m " + doTable.Group + "Model." + modelName + "\n"
	}
	pageParams := "func buildParamsPage" + modelName + "(params *params." + modelName + "PageParams) []page.Param {\n"
	pageParams += "\tvar daoParams []page.Param\n"
	pageParams += "\t//Todo 根据实际情况拼分页查询条件\n"
	if doTable.Group == "" {
		pageParams += "\tvar column = &model." + modelName + "Columns\n"
	} else {
		pageParams += "\tvar column = &" + doTable.Group + "Model." + modelName + "Columns\n"
	}

	var pks []TableField

	for _, field := range fields {
		//字段名
		fieldName := strcase.ToCamel(strings.ToLower(field.FieldName))
		buildParams := false
		if strings.Contains(doTable.PK, field.FieldName) {
			buildParams = true
			if !doTable.AutoGenerateID {
				addOne += "\tm." + fieldName + "=addParams." + fieldName + "\n"
			}
			updateByPrimaryKey += "\tm." + fieldName + "=updateParams." + fieldName + "\n"
			pks = append(pks, field)
		} else if strings.ToLower(field.FieldName) == Handler {
			addOne += "\tcurrentUser := request.GetCurrentUser(c)\n"
			addOne += "\tvar handler = fmt.Sprintf(\"%s(%s)\",currentUser.Name,currentUser.Account)\n"
			addOne += "\tm.Handler = &handler\n"
			updateByPrimaryKey += "\tcurrentUser := request.GetCurrentUser(c)\n"
			updateByPrimaryKey += "\tvar handler = fmt.Sprintf(\"%s(%s)\",currentUser.Name,currentUser.Account)\n"
			updateByPrimaryKey += "\tm.Handler = &handler\n"
		} else if strings.ToLower(field.FieldName) == CreateTime ||
			strings.ToLower(field.FieldName) == UpdateTime {
		} else {
			addOne += "\tm." + fieldName + "=addParams." + fieldName + "\n"
			updateByPrimaryKey += "\tm." + fieldName + "=updateParams." + fieldName + "\n"
			buildParams = true
		}
		if buildParams {
			pageParams +=
				"\tif params." + fieldName + " != nil  {\n" +
					"\t\tdaoParams = append(daoParams, page.Param{Column: column." + fieldName + ", Op: sqlOp.Equal, Value: params." + fieldName + "})\n" +
					"\t}\n"
		}
	}

	if doTable.Group == "" {
		addOne += "\tdao.Create" + modelName + "(&m)\n"
	} else {
		addOne += "\t" + doTable.Group + "Dao.Create" + modelName + "(&m)\n"
	}
	addOne += "}\n"
	if doTable.Group == "" {
		updateByPrimaryKey += "\tdao.Update" + modelName + "ByPrimaryKey(&m)\n"
	} else {
		updateByPrimaryKey += "\t" + doTable.Group + "Dao.Update" + modelName + "ByPrimaryKey(&m)\n"
	}
	updateByPrimaryKey += "}\n"
	findByPrimaryKey := ""
	if len(pks) > 0 {
		pkParams := ""
		pkFields := ""
		for j := 0; j < len(pks); j++ {
			field := pks[j]
			pkParams += strcase.ToLowerCamel(strings.ToLower(field.FieldName)) + " " + getModelFiledType(field) + ", "
			pkFields += strcase.ToLowerCamel(strings.ToLower(field.FieldName)) + ", "
		}
		pkParams = pkParams[0 : len(pkParams)-2]
		pkFields = pkFields[0 : len(pkFields)-2]
		if doTable.Group == "" {
			findByPrimaryKey += "func Find" + modelName + "(" + pkParams + ") *model." + modelName + " {\n"
			findByPrimaryKey += "\treturn dao.Find" + modelName + "ByPrimaryKey(" + pkFields + ")\n"
		} else {
			findByPrimaryKey += "func Find" + modelName + "(" + pkParams + ") *" + doTable.Group + "Model." + modelName + " {\n"
			findByPrimaryKey += "\treturn " + doTable.Group + "Dao.Find" + modelName + "ByPrimaryKey(" + pkFields + ")\n"
		}
		findByPrimaryKey += "}\n"
	}
	pageQuery := "func Page" + modelName + "(params *params." + modelName + "PageParams) *page.Data {\n"

	if doTable.Group == "" {
		pageQuery += "\treturn dao.Page" + modelName + "(buildParamsPage" + modelName + "(params),params.PageNum,params.PageSize)\n"
	} else {
		pageQuery += "\treturn " + doTable.Group + "Dao.Page" + modelName + "(buildParamsPage" + modelName + "(params),params.PageNum,params.PageSize)\n"
	}

	pageQuery += "}\n"

	pageParams += "\treturn daoParams\n"
	pageParams += "}"
	fileContent := ""
	if len(pks) > 0 {
		fileContent = packageStr + importStr + addOne + updateByPrimaryKey + findByPrimaryKey + pageQuery + pageParams
	} else {
		fileContent = packageStr + importStr + addOne + pageQuery + pageParams
	}
	createOrReplaceFile(BasePackage, servicePackageName, doTable.Group, "", modelName, fileContent)
}

//生成Api相关Response
func generateResponse(table *Table, doTable TableInfo) {
	projectName := ProjectName
	modelName := getModelName(table.Name, doTable)
	packageStr := "package response\n\n"

	importStr := "import (\n"
	importStr += "\t\"" + projectName + "/a/getWay\"\n"
	importStr += "\t\"" + projectName + "/a/page\"\n\n"
	if doTable.Group == "" {
		importStr += "\t\"" + projectName + "/" + BasePackage + "/" + modelPackageName + "\"\n"
	} else {
		importStr += "\t" + doTable.Group + "Model \"" + projectName + "/" + BasePackage + "/" + modelPackageName + "/" + doTable.Group + "\"\n"
	}
	importStr += ")\n"

	pageResponse := "type " + modelName + "PageResponse struct {\n"
	pageResponse += "\tgetWay.ServerResponse\n"
	pageResponse += "\tData " + modelName + "PageData       `json:\"data\"`\n"
	pageResponse += "}\n\n"
	pageResponse += "type " + modelName + "PageData struct {\n"
	pageResponse += "\tpage.Page\n"
	if doTable.Group == "" {
		pageResponse += "\tList []" + "model." + modelName + "\n"
	} else {
		pageResponse += "\tList []" + doTable.Group + "Model." + modelName + "\n"
	}

	pageResponse += "}\n"

	fileContent := packageStr + importStr + pageResponse
	createOrReplaceFile(BasePackage, webPackageName, doTable.Group, "response", modelName, fileContent)
}

func getApiPrefix(tableName string, ignoreRouterPrefixL int) string {
	apiPrefix := "/" + BasePackage
	reg := regexp.MustCompile(`(_)`)
	tableApiPrefix := reg.ReplaceAllLiteralString(strings.ToLower(tableName), "/")
	tableApiPrefix = tableApiPrefix[ignoreRouterPrefixL:]
	if strings.Index(tableApiPrefix, "/") != 0 {
		tableApiPrefix = "/" + tableApiPrefix
	}
	apiPrefix = apiPrefix + tableApiPrefix
	return apiPrefix
}

//生成api
func generateApi(table *Table, fields []TableField, doTable TableInfo) {
	projectName := ProjectName
	modelName := getModelName(table.Name, doTable)
	packageStr := "package " + webPackageName + "\n\n"
	haveHandler := false
	var pks []TableField
	for _, field := range fields {
		if strings.Contains(doTable.PK, field.FieldName) {
			pks = append(pks, field)
		}
		if strings.ToLower(field.FieldName) == Handler {
			haveHandler = true
		}
	}
	importStr := "import (\n"
	importStr += "\t\"" + projectName + "/a/exception\"\n"
	importStr += "\t\"" + projectName + "/a/exception/catch\"\n"
	importStr += "\t\"" + projectName + "/a/getWay\"\n"
	if doTable.Group == "" {
		importStr += "\t\"" + projectName + "/" + BasePackage + "/" + servicePackageName + "\"\n"
		importStr += "\t\"" + projectName + "/" + BasePackage + "/" + webPackageName + "/params\"\n"
	} else {
		importStr += "\t" + doTable.Group + "Service \"" + projectName + "/" + BasePackage + "/" + servicePackageName + "/" + doTable.Group + "\"\n"
		importStr += "\t\"" + projectName + "/" + BasePackage + "/" + webPackageName + "/" + doTable.Group + "/params\"\n"
	}
	importStr += "\t\"github.com/gin-gonic/gin\"\n"
	importStr += ")\n"

	apiPrefix := getApiPrefix(table.Name, doTable.IgnoreRouterPrefixL)
	addRouter := apiPrefix + "/add"
	updateRouter := apiPrefix + "/update"
	pageRouter := apiPrefix + "/page"
	initRouter := "// Init" + modelName + "Rout 在InitRouter中添加Init" + modelName + "Rout(r),进行路由初始化\n"
	initRouter += "func Init" + modelName + "Rout(engine *gin.Engine) {\n"
	initRouter += "\tengine.POST(\"" + addRouter + "\", add" + modelName + ")\n"
	if len(pks) > 0 {
		initRouter += "\tengine.POST(\"" + updateRouter + "\", update" + modelName + ")\n"
	}
	initRouter += "\tengine.POST(\"" + pageRouter + "\", page" + modelName + ")\n"
	initRouter += "}\n"

	addApi := "// @Tags " + table.Description + "\n"
	addApi += "// @Summary  新增" + table.Description + "\n"
	addApi += "// @Param " + modelName + "AddParams body params." + modelName + "AddParams false \"参数\"\n"
	addApi += "// @Param token header string true \"登陆token\"\n"
	addApi += "// @Product json\n"
	addApi += "// @Success 200 {object} getWay.ServerResponse\n"
	addApi += "// @Router " + addRouter + " [POST]\n"
	addApi += "func add" + modelName + "(c *gin.Context) {\n"
	addApi += "\tdefer catch.ExceptionCatch(c)\n"
	addApi += "\tvar param params." + modelName + "AddParams\n"
	addApi += "\tvar err error\n"
	addApi += "\terr = getWay.BindRequestBody(c, &param)\n"
	addApi += "\tif err==nil {\n"
	addPrams := ""
	if haveHandler {
		addPrams = "(param, c)"
	} else {
		addPrams = "(param)"
	}
	if doTable.Group == "" {
		addApi += "\t\tservice.Add" + modelName + addPrams + "\n"
	} else {
		addApi += "\t\t" + doTable.Group + "Service.Add" + modelName + "(param)\n"
	}
	addApi += "\t\tgetWay.Success(c)\n"
	addApi += "\t}else{\n"
	addApi += "\t\texception.ThrowsErr(err)\n"
	addApi += "\t}\n"
	addApi += "}\n"

	updateApi := "// @Tags " + table.Description + "\n"
	updateApi += "// @Summary  修改" + table.Description + "\n"
	updateApi += "// @Param " + modelName + "AddParams body params." + modelName + "UpdateParams false \"修改参数\"\n"
	updateApi += "// @Param token header string true \"登陆token\"\n"
	updateApi += "// @Product json\n"
	updateApi += "// @Success 200 {object} getWay.ServerResponse\n"
	updateApi += "// @Router " + updateRouter + " [POST]\n"
	updateApi += "func update" + modelName + "(c *gin.Context) {\n"
	updateApi += "\tdefer catch.ExceptionCatch(c)\n"
	updateApi += "\tvar param params." + modelName + "UpdateParams\n"
	updateApi += "\tvar err error\n"
	updateApi += "\terr = getWay.BindRequestBody(c, &param)\n"
	updateApi += "\tif err==nil {\n"
	updatePrams := ""
	if haveHandler {
		updatePrams = "(param, c)"
	} else {
		updatePrams = "(param)"
	}
	if doTable.Group == "" {
		updateApi += "\t\tservice.Update" + modelName + updatePrams + "\n"
	} else {
		updateApi += "\t\t" + doTable.Group + "Service.Update" + modelName + updatePrams + "\n"
	}
	updateApi += "\t\tgetWay.Success(c)\n"
	updateApi += "\t}else{\n"
	updateApi += "\t\texception.ThrowsErr(err)\n"
	updateApi += "\t}\n"
	updateApi += "}\n"

	pageApi := "// @Tags " + table.Description + "\n"
	pageApi += "// @Summary  分页查询" + table.Description + "\n"
	pageApi += "// @Param " + modelName + "PageParams body params." + modelName + "PageParams false \"参数\"\n"
	pageApi += "// @Param token header string true \"登陆token\"\n"
	pageApi += "// @Product json\n"
	pageApi += "// @Success 200 {object} response." + modelName + "PageResponse\n"
	pageApi += "// @Router " + pageRouter + " [POST]\n"
	pageApi += "func page" + modelName + "(c *gin.Context) {\n"
	pageApi += "\tdefer catch.ExceptionCatch(c)\n"
	pageApi += "\tvar param params." + modelName + "PageParams\n"
	pageApi += "\tvar err error\n"
	pageApi += "\terr = getWay.BindRequestBody(c, &param)\n"
	pageApi += "\tif err==nil {\n"
	if doTable.Group == "" {
		pageApi += "\t\tpage := service.Page" + modelName + "(&param)\n"
	} else {
		pageApi += "\t\tpage := " + doTable.Group + "Service.Page" + modelName + "(param)\n"
	}
	pageApi += "\t\tgetWay.SuccessData(c,*page)\n"
	pageApi += "\t}else{\n"
	pageApi += "\t\texception.ThrowsErr(err)\n"
	pageApi += "\t}\n"
	pageApi += "}\n"

	fileContent := ""
	if len(pks) > 0 {
		fileContent = packageStr + importStr + initRouter + addApi + updateApi + pageApi
	} else {
		fileContent = packageStr + importStr + initRouter + addApi + pageApi
	}
	createOrReplaceFile(BasePackage, webPackageName, doTable.Group, "", modelName, fileContent)

}

//检查文件是否存在
func checkFileIsExist(filename string) bool {
	var exist = true
	if _, err := os.Stat(filename); os.IsNotExist(err) {
		exist = false
	}
	return exist
}

func buildFileDirectory(basePackageName, packageName, group, subPackageName string) string {
	fileDirectory := ""
	if basePackageName != "" {
		fileDirectory += basePackageName + PathSeparator
	}
	if packageName != "" {
		fileDirectory += packageName + PathSeparator
	}
	if &group != nil && group != "" {
		fileDirectory += group + PathSeparator
	}
	fileDirectory += subPackageName + PathSeparator
	return fileDirectory
}

func createOrReplaceFile(basePackage, packageName, group, subPackageName, modelName, fileContent string) {
	fileSuffix := ""
	if subPackageName != "" {
		fileSuffix = strcase.ToCamel(subPackageName)
	} else {
		fileSuffix = strcase.ToCamel(packageName)
	}
	fileDirectory := buildFileDirectory(basePackage, packageName, group, subPackageName)
	filename := fileDirectory + modelName + fileSuffix + ".go"
	var f *os.File
	var err error
	if checkFileIsExist(filename) {
		if !ModelReplace {
			fmt.Println(modelName + " 已存在，需删除才能重新生成...")
			return
		}
		f, err = os.OpenFile(filename, os.O_WRONLY|os.O_TRUNC, os.ModePerm) //打开文件
		if err != nil {
			panic(err)
		}
	} else {
		//创建目录
		os.MkdirAll(fileDirectory, os.ModePerm)
		f, err = os.OpenFile(filename, os.O_CREATE|os.O_EXCL, os.ModePerm)
		if err != nil {
			panic(err)
		}
	}
	defer f.Close()
	_, err = io.WriteString(f, fileContent)
	fmt.Println(modelName + fileSuffix + " 已生成...")
}
