package getWay

import (
	"encoding/json"
	"fmt"
	"git.168cad.top/zhengqiuyun/rym-util/a/conf"
	"git.168cad.top/zhengqiuyun/rym-util/a/db/redis"
	"git.168cad.top/zhengqiuyun/rym-util/a/log"
	"github.com/gin-gonic/gin"
	"io/ioutil"
	"net"
	"net/http"
	"strings"
	"time"
)

func GetCurrentUserId(c *gin.Context) (uint32, bool) {
	userId, b := c.Get(CurrentUserIdKey)
	if b {
		transUserId := userId.(uint32)
		return transUserId, b
	}
	return 0, b
}

type UserInfo struct {
	Id      uint32
	Account string
	Name    string
}

func GetCurrentUser(c *gin.Context) *UserInfo {
	var userInfo = UserInfo{}
	adminResponse, b := c.Get(CurrentUserKey)
	if b {
		admin := adminResponse.(Admin)
		userInfo.Name = admin.Name
		userInfo.Account = admin.LoginAccount
		userInfo.Id = admin.Id
	}
	return &userInfo
}

func GetAdminInfo(token string) Admin {
	var adminResponse = AdminResponse{}
	resp, err := http.Get(conf.SystemHost + "/admin/current?token=" + token)
	if err != nil {
		log.Error(err.Error())
		return Admin{}
	}
	defer resp.Body.Close()
	adminResponseBytes, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		log.Error(err.Error())
		return Admin{}
	}
	err = json.Unmarshal(adminResponseBytes, &adminResponse)
	if err != nil {
		log.Error(err.Error())
	}
	return adminResponse.Admin
}

type AdminResponse struct {
	code  int
	msg   string
	Admin Admin
}

type Admin struct {
	Id           uint32
	Name         string
	LoginAccount string
}

const requestBodyKey = "requestBody"

func BindRequestBody(c *gin.Context, param interface{}) error {
	c.ShouldBindJSON(param)
	bytes, err := json.Marshal(param)
	c.Set(requestBodyKey, string(bytes))
	return err
}

func GetParams(c *gin.Context) string {
	rd := ""
	params := c.Request.URL.Query().Encode()
	if params != "" {
		rd += params
	}
	ad := c.GetString(requestBodyKey)
	if ad != "" {
		rd += ad
	}
	return rd
}

func GetServerIP() string {
	addrs, err := net.InterfaceAddrs()
	if err != nil {
		return ""
	}
	for _, addr := range addrs {
		ipAddr, ok := addr.(*net.IPNet)
		if !ok {
			continue
		}
		if ipAddr.IP.IsLoopback() {
			continue
		}
		if !ipAddr.IP.IsGlobalUnicast() {
			continue
		}
		return ipAddr.IP.String()
	}
	return ""
}

func GetClientIP(c *gin.Context) string {
	ip := c.ClientIP()
	return ip
}

func GetURL(c *gin.Context) string {
	url := c.Request.URL.Path
	return url
}
func GetMethod(c *gin.Context) string {
	method := c.Request.Method
	return method
}
func SetStartTime(c *gin.Context) {
	c.Set(StartTimeKey, time.Now().UnixNano())
	logStart(c)
}
func GetStartTime(c *gin.Context) int64 {
	startTime := c.GetInt64(StartTimeKey)
	return startTime
}

func ValidRequest(c *gin.Context) {
	defer exceptionGetWayCatch(c)
	ip := c.ClientIP()
	url := c.Request.RequestURI
	token := GetToken(c)
	if token == "" {
		if checkIp(ip) { //如果没有token，看是否是ip白名单
			log.Debug(fmt.Sprintf("IP白名单校验通过：%s", ip))
		} else {
			log.Debug(fmt.Sprintf("IP白名单校验不通过：%s", ip))
			if checkUri(url) { //如果不是ip白名单，看是否是url白名单，例如一些第三方的回调
				log.Debug(fmt.Sprintf("URL白名单校验通过：%s", url))
			} else {
				log.Debug(fmt.Sprintf("URL白名单校验不通过：%s", url))
				ThrowsGetWayErr(Illegal9)
			}
		}
	} else {
		if redis.Exists(token) { //校验token是否有效
			currentUser := GetAdminInfo(token)
			c.Set(CurrentUserKey, currentUser)
			log.Debug(fmt.Sprintf("登陆校验通过：%s,对应用户id: %d", token, currentUser.Id))
		} else {
			log.Debug(fmt.Sprintf("登陆校验不通过：%s", token))
			ThrowsGetWayErr(Expire)
		}
	}
}

func exceptionGetWayCatch(c *gin.Context) {
	err := recover() //获取异常
	if err != nil {
		switch err.(type) {
		case Error:
			e := (err).(Error)
			FailGetWay(c, e.Code)
			break
		}
	}
}

func checkUri(uri string) bool {
	for _, v := range conf.WhiteListUri {
		if strings.Index(uri, v) == 0 {
			return true
		}
	}
	return false
}

func checkIp(ip string) bool {
	for _, v := range conf.WhiteListIp {
		if strings.Index(ip, v) == 0 {
			return true
		}
	}
	return false
}

func logStart(c *gin.Context) {
	log.Info(fmt.Sprintf("开始 %s %s %s %s", GetMethod(c), GetURL(c), GetToken(c), GetClientIP(c)))
}
