package lib_allinpay

import (
	"crypto"
	"crypto/md5"
	"crypto/rsa"
	"encoding/base64"
	"encoding/json"
	"fmt"
	"git.168cad.top/zhengqiuyun/rym-util/a/conf"
	"git.168cad.top/zhengqiuyun/rym-util/a/log"
	"git.168cad.top/zhengqiuyun/rym-util/exception"
	"io"
	"io/ioutil"
	"net/http"
	"net/url"
	"sort"
	"strings"
	"time"
)

type Client struct {
	httpClient  *http.Client
	privateKey  *rsa.PrivateKey
	tlPublicKey *rsa.PublicKey
}

func New(privateKeyPath, pwd, tlPublicKeyPath string) (*Client, error) {
	var client = Client{}
	privateKey, err := getPrivateKey(privateKeyPath, pwd)
	if err != nil {
		return nil, err
	}
	client.privateKey = privateKey
	publicKey, err := loadPublicKeyWithPath(tlPublicKeyPath)
	if err != nil {
		return nil, err
	}
	client.tlPublicKey = publicKey
	client.httpClient = http.DefaultClient
	return &client, nil
}

func (c *Client) doRequest(method string, param Param, result interface{}) (err error) {
	var buf io.Reader
	requestStr := ""
	if param != nil {
		p, err := c.URLValues(param)
		if err != nil {
			return err
		}
		requestStr = p.Encode()
		buf = strings.NewReader(requestStr)
	}
	log.Debug(fmt.Sprintf("请求参数：%s", requestStr))
	req, err := http.NewRequest(method, conf.AllinpayUrl, buf)
	if err != nil {
		return err
	}
	req.Header.Set("Content-Type", "application/x-www-form-urlencoded;charset=utf-8")
	resp, err := c.httpClient.Do(req)
	if resp != nil {
		defer resp.Body.Close()
	}
	if err != nil {
		return err
	}
	data, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return err
	}
	log.Debug(fmt.Sprintf("请求结果：%s", string(data)))
	ok, err := verifyData(data, c.tlPublicKey)
	if err != nil {
		return err
	}
	if !ok {
		exception.ThrowsErrS("请求结果校验错误，请检查公钥配置")
	}
	err = json.Unmarshal(data, result)
	if err != nil {
		return err
	}
	return err
}

func verifyData(data []byte, key *rsa.PublicKey) (ok bool, err error) {
	sign := ""
	dataMap := make(map[string]interface{})
	err = json.Unmarshal(data, &dataMap)
	var sortKeys = util.SortKeys(dataMap)
	signDataMap := make(map[string]interface{})
	for _, k := range sortKeys {
		if k == "sign" {
			sign = dataMap[k].(string)
		} else {
			signDataMap[k] = dataMap[k]
		}
	}
	signDataByte, _ := json.Marshal(signDataMap)
	md5Str := md5B(signDataByte)
	signBytes, err := base64.StdEncoding.DecodeString(sign)
	if err != nil {
		return false, err
	}
	if err = RSAVerifyWithKey([]byte(md5Str), signBytes, key, crypto.SHA256); err != nil {
		return false, err
	}
	return true, nil
}

func (c *Client) URLValues(param Param) (value url.Values, err error) {
	var p = url.Values{}
	p.Add("appId", conf.AllinpayAppId)
	p.Add("method", param.apiName())
	p.Add("format", "JSON")
	p.Add("charset", "utf-8")
	p.Add("timestamp", util.FormatDateTime(time.Now()))
	p.Add("version", "1.0")
	bytes, err := json.Marshal(param)
	if err != nil {
		return nil, err
	}
	p.Add("bizContent", string(bytes))
	sign, err := signWithPKCS1v15(p, c.privateKey, crypto.SHA256)
	if err != nil {
		return nil, err
	}
	p.Add("sign", sign)
	p.Add("signType", "SHA256WithRSA")
	return p, nil
}

func signWithPKCS1v15(param url.Values, privateKey *rsa.PrivateKey, hash crypto.Hash) (s string, err error) {
	if param == nil {
		param = make(url.Values, 0)
	}
	var pList = make([]string, 0, 0)
	for key := range param {
		var value = strings.TrimSpace(param.Get(key))
		if len(value) > 0 {
			pList = append(pList, key+"="+value)
		}
	}
	sort.Strings(pList)
	var src = strings.Join(pList, "&")
	log.Debug(fmt.Sprintf("签名数据：%s", src))
	md5Str := md5S(src)
	sig, err := RSASignWithKey([]byte(md5Str), privateKey, hash)
	if err != nil {
		return "", err
	}
	s = base64.StdEncoding.EncodeToString(sig)
	return s, nil
}

func md5B(src []byte) string {
	h := md5.New()
	h.Write(src)
	bb := h.Sum(nil)
	sEnc := base64.StdEncoding.EncodeToString(bb)
	return sEnc
}

func md5S(src string) string {
	return md5B([]byte(src))
}

func (c *Client) GetTradeNotification(data map[string]string, sign string) (result PayNotifyData, err error) {
	err = c.getDefaultNotification(data, sign, &result)
	return result, err
}

func (c *Client) getDefaultNotification(data map[string]string, sign string, result interface{}) error {
	var pList = make([]string, 0, 0)
	for k, v := range data {
		pList = append(pList, k+"="+v)
	}
	sort.Strings(pList)
	var src = strings.Join(pList, "&")
	log.Debug(fmt.Sprintf("签名数据：%s", src))
	md5Str := md5B([]byte(src))
	signBytes, err := base64.StdEncoding.DecodeString(sign)
	if err != nil {
		return err
	}
	if err = RSAVerifyWithKey([]byte(md5Str), signBytes, c.tlPublicKey, crypto.SHA256); err != nil {
		return err
	}
	err = json.Unmarshal([]byte(data["bizContent"]), result)
	if err != nil {
		return err
	}
	return err
}
